

#include <arpa/inet.h>
#include <errno.h>
#include <fcntl.h>
#include <fstream>
#include <iostream>
#include <netdb.h>
#include <netinet/in.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <vector>

#include "json.hpp"
#include "video_processor.h"
using json = nlohmann::json;

#define CONFIG_JSON_FILE "./configs/video.json"

int main(int argc, char **argv) {
    int ret = 0;
    json cfg_json;
    {
        std::string path;
        if (argc > 1) {
            path = argv[1];
        } else {
            path = CONFIG_JSON_FILE;
        }

        std::ifstream is(path);
        if (!is.good()) {
            std::cout << "Error: video config file path:[" << path
                      << "] not find ." << std::endl;
            return -1;
        }

        is >> cfg_json;
        std::cout << "video config file path:" << path << "\n json:" << cfg_json
                  << std::endl;
    }

    ret = video_processor_init();
    if (ret != 0) {
        printf("VideoProcessorInit failure .\n");
        return -1;
    }
    printf("VideoProcessorInit Sucess .\n");
    json video_source_array = cfg_json["videoSource"];
    for (int i = 0; i < video_source_array.size(); i++) {

        // printf("ch %d,rtsp
        // %s\n",video_source_array[i]["ch"].get<int>(),video_source_array[i]["rtsp"].get<std::string>().c_str());
        ret = video_processor_set_source(
            video_source_array[i]["ch"].get<int>(),
            video_source_array[i]["rtsp"].get<std::string>().c_str());
        if (ret != 0) {
            printf("SetSource failure .\n");
        } else {
            printf("SetSource Sucess .\n");
        }
    }

    if (cfg_json["frameRate"] != nullptr) {
        ret = video_processor_set_frame_rate(
            (VideoProcessorFrameRate_t)cfg_json["frameRate"].get<int>());
        if (ret != 0) {
            printf("SetFrameRate failure .\n");
        } else {
            printf("SetFrameRate Sucess .\n");
        }
    }

    if (cfg_json["netWork"] != nullptr) {

        char mac_addr[100] = {0};
        json network = cfg_json["netWork"];
        NetWorkConfigParam_t network_param = {nullptr, nullptr, nullptr,
                                              nullptr};
        strcpy(mac_addr, network["mac"].get<std::string>().c_str());
        network_param.mac = mac_addr;
        network_param.gw = network["gw"].get<std::string>().c_str();
        network_param.ip = network["ip"].get<std::string>().c_str();
        network_param.netmask = network["netmask"].get<std::string>().c_str();

        ret = video_processor_set_network(&network_param);
        if (ret != 0) {
            printf("SetNetWork failure .\n");
        } else {
            printf("SetNetWork Sucess .\n");
        }
    }

    if (cfg_json["channelSequence"] != nullptr) {
        ret = video_processor_set_channel_sequence(
            cfg_json["channelSequence"].get<std::string>().c_str());
        if (ret != 0) {
            printf("Channel Sequence failed.\n");
        } else {
            printf("Channel Sequence Sucess .\n");
        }
    }

    video_processor_deinit();
    return 0;
}