#pragma once
#include <iostream>
#include <linux/videodev2.h>
#include <stdint.h>
#include <thread>

typedef int (*read_cb_t)(uint8_t ch_num, uint32_t frame_num, void *pdata,
                         size_t size, int width, int height, void *user_data);

class MultiChannelCamera {
  public:
    struct CameraConfig {
        int fps;
        int width;
        int height;
        void *user_data;
        read_cb_t cb;
    };
    void setConfig(CameraConfig &config);
    int start(void);
    int stop(void);
    MultiChannelCamera(std::string dev_name, read_cb_t cb, void *user_data)
        : _dev_name(dev_name), _cb_read(cb), _user_data(user_data) {
        _fps = 30;
        _width = 1920;
        _height = 1080;
        common_init();
    };

    MultiChannelCamera(std::string dev_name, read_cb_t cb, void *user_data,
                       int fps, int width, int height)
        : _dev_name(dev_name), _cb_read(cb), _user_data(user_data), _fps(fps),
          _width(width), _height(height) {
        common_init();
    };

    ~MultiChannelCamera(){};

  private:
    std::string _dev_name;

    void *_user_data;
    read_cb_t _cb_read;
    int _fps;
    int _width;
    int _height;

    int _fd;

    bool _loop;
    std::thread _worker_thread;

    v4l2_buffer _last_buf;

    struct buffer_t {
        void *start;
        size_t length;
    };
    buffer_t *_pbuffers;
    int _buffers_count;
    int _buffers_size;

    uint8_t *_empty_save_buf;
    void run(void);

    void common_init(void) {
        _buffers_count = 4;
        _loop = false;
        _fd = -1;
        _pbuffers = nullptr;
        _buffers_size = 0;
        _empty_save_buf = nullptr;
        _last_buf.m.userptr = 0;
    };
    int process_image(void *p, int size);
    int read_frame(void);

    int stop_capturing(void);
    int start_capturing(void);

    int init_userp(uint32_t buffer_size);

    int init_device(void);
    void close_device(void);
    int open_device(void);
    int release_device(void);
};
