#ifndef VIDEO_PROCESSOR_PROTOCAL_H
#define VIDEO_PROCESSOR_PROTOCAL_H
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <stdint.h>

typedef enum VideoProcessorFrameRate {
    VIDEO_PROCESSOR_FRAMERATE_P24 = 24,
    VIDEO_PROCESSOR_FRAMERATE_P30 = 30,
} VideoProcessorFrameRate_t;

/*
举例说明：
* mac:      "23:50:95:da:65:fe"
* ip:       "192.168.1.125"
* netmask:  "255.255.255.0"
* gw:       "192.168.1.1"
*/
typedef struct NetWorkConfigParam {
    const char *mac;
    const char *ip;
    const char *netmask;
    const char *gw;
} NetWorkConfigParam_t;

/*
 * 必须首先被调用
 */
int video_processor_init(void);
void video_processor_deinit(void);

/*
 * Function:设置 hdmi通道的帧率
 * frame_rate: 目前只支持 P24 与 P30
 */
int video_processor_set_frame_rate(VideoProcessorFrameRate_t frame_rate);

/*
 * Function: 设置通道 RTSP流地址
 * ch： 范围 0-15
 * rtsp: "rtsp://admin:admin123456@192.168.1.64:554//Streaming/Channels/1>"
 */
int video_processor_set_source(uint16_t ch, const char *rtsp);

/*
* Function: 设置Video Processor 网络参数
*  network_cfg: 见NetWorkConfigParam_t
*/

int video_processor_set_network(NetWorkConfigParam_t *network_cfg);

/*
 * Function: 设置通道时序
 * seq: <ch1:2,ch2:3,ch15:2,ch4:1>
 *      举例说明：控制hdmi 通道按照 如下顺序发送视频帧：
 *      ch1,ch1,ch2,ch2,ch2,ch15,ch15,ch4 ..... 重复这个顺序
 *          ^        ^          ^       ^
 *          |        |          |       |
 *      |-ch1:2|----ch2:3--|-ch15:2--| ch4|
 */
int video_processor_set_channel_sequence(const char *seq);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif

